import { CommandMetadata, CommandMetadataInput, CommandMetadataOption, HydratedCommandMetadata, HydratedNamespaceMetadata, ICommand, INamespace, NamespaceLocateResult, NamespaceMetadata } from '../definitions';
import { Colors } from './colors';
export declare abstract class HelpFormatter {
    protected readonly colors: Colors;
    constructor({colors}: {
        colors?: Colors;
    });
    abstract format(): Promise<string>;
}
export interface NamespaceHelpFormatterDeps<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> {
    readonly location: NamespaceLocateResult<C, N, M, I, O>;
    readonly namespace: N;
    readonly colors?: Colors;
}
export declare class NamespaceHelpFormatter<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> extends HelpFormatter {
    protected readonly location: NamespaceLocateResult<C, N, M, I, O>;
    protected readonly namespace: N;
    protected readonly dotswidth: number;
    protected _metadata?: NamespaceMetadata;
    protected _fullName?: string;
    constructor({location, namespace, colors}: NamespaceHelpFormatterDeps<C, N, M, I, O>);
    /**
     * Given command metadata, decide whether to keep or discard the command that
     * the metadata represents.
     *
     * @param meta: The metadata of the command.
     * @return `true` to keep, `false` to discard
     */
    filterCommandCallback?(meta: HydratedCommandMetadata<C, N, M, I, O>): Promise<boolean>;
    /**
     * Insert text that appears before a commands's summary.
     *
     * @param meta: The metadata of the command.
     */
    formatBeforeCommandSummary?(meta: HydratedCommandMetadata<C, N, M, I, O>): Promise<string>;
    /**
     * Insert text before summaries of listed subnamespaces.
     *
     * @param meta The metadata of the namespace.
     * @param commands An array of the metadata of the namespace's commands.
     */
    formatBeforeNamespaceSummary?(meta: HydratedNamespaceMetadata, commands: ReadonlyArray<HydratedCommandMetadata<C, N, M, I, O>>): Promise<string>;
    /**
     * Insert text before the namespace's summary.
     */
    formatBeforeSummary?(): Promise<string>;
    /**
     * Insert text after the namespace's summary.
     */
    formatAfterSummary?(): Promise<string>;
    getNamespaceMetadata(): Promise<NamespaceMetadata>;
    getNamespaceFullName(): Promise<string>;
    formatHeader(): Promise<string>;
    formatSummary(): Promise<string>;
    formatDescription(): Promise<string>;
    getExtraOptions(): Promise<string[]>;
    formatUsage(): Promise<string>;
    formatCommands(): Promise<string>;
    formatCommandGroup(title: string, commands: ReadonlyArray<HydratedCommandMetadata<C, N, M, I, O>>): Promise<string>;
    getListOfCommandDetails(commands: ReadonlyArray<HydratedCommandMetadata<C, N, M, I, O>>): Promise<string[]>;
    getListOfNamespaceDetails(commands: ReadonlyArray<HydratedCommandMetadata<C, N, M, I, O>>): Promise<string[]>;
    /**
     * Insert text that appears after a commands's summary.
     *
     * @param meta: The metadata of the command.
     */
    formatAfterCommandSummary(meta: HydratedCommandMetadata<C, N, M, I, O>): Promise<string>;
    /**
     * Insert text that appears after a namespace's summary.
     *
     * @param meta The metadata of the namespace.
     * @param commands An array of the metadata of the namespace's commands.
     */
    formatAfterNamespaceSummary(meta: HydratedNamespaceMetadata, commands: ReadonlyArray<HydratedCommandMetadata<C, N, M, I, O>>): Promise<string>;
    format(): Promise<string>;
}
export interface CommandHelpFormatterDeps<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> {
    readonly location: NamespaceLocateResult<C, N, M, I, O>;
    readonly command: C;
    readonly colors?: Colors;
}
export declare class CommandHelpFormatter<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> extends HelpFormatter {
    protected readonly location: NamespaceLocateResult<C, N, M, I, O>;
    protected readonly command: C;
    protected readonly dotswidth: number;
    protected _metadata?: M;
    protected _fullName?: string;
    constructor({location, command, colors}: CommandHelpFormatterDeps<C, N, M, I, O>);
    /**
     * Given an option definition from command metadata, decide whether to keep
     * or discard it.
     *
     * @return `true` to keep, `false` to discard
     */
    filterOptionCallback?(option: O): Promise<boolean>;
    /**
     * Insert text that appears before an option's summary.
     *
     * @param opt The metadata of the option.
     */
    formatBeforeOptionSummary?(opt: O): Promise<string>;
    /**
     * Insert text before the command's summary.
     */
    formatBeforeSummary?(): Promise<string>;
    /**
     * Insert text after the command's summary.
     */
    formatAfterSummary?(): Promise<string>;
    getCommandMetadata(): Promise<M>;
    getCommandFullName(): Promise<string>;
    formatHeader(): Promise<string>;
    formatSummary(): Promise<string>;
    formatDescription(): Promise<string>;
    formatInlineInput(input: I): Promise<string>;
    formatUsage(): Promise<string>;
    formatInputs(): Promise<string>;
    formatOptionLine(opt: O): Promise<string>;
    formatAfterOptionSummary(opt: O): Promise<string>;
    formatOptionDefault(opt: O): Promise<string>;
    formatOptions(): Promise<string>;
    formatOptionsGroup(title: string, options: O[]): Promise<string>;
    formatExamples(): Promise<string>;
    format(): Promise<string>;
}
